
import tester.*;

/**
 * Testing class StringListTest.
 * 
 * @author Stephen Bloch
 * @version v3, 3 Mar 2009
 */
public class StringListTest
{
    // Instance variables go here, typically examples to test.
    // For example, if you're testing the Date class, you might say
    // Date myBirthday = new Date(1964, 1, 27);
    // Date millennium = new Date(2001, 1, 1);
    private ESL esl = new ESL();
    private NESL aList = new NESL("Bob", new NESL("Phyllis", this.esl));
    private StringList mt = new ESL();
    private StringList a = new NESL("a",this.mt);
    private StringList b = new NESL("b",this.mt);
    private StringList ab = new NESL("a",this.b);
    private StringList abc = new NESL("a", new NESL("b", new NESL("c", this.mt)));
    private StringList plainPizza = new ESL();
    private StringList justSauce = this.plainPizza.add("sauce");
    private StringList sauceNCheese = this.justSauce.add("cheese");
    private StringList pepPizza = this.sauceNCheese.add("pepperoni");
    private StringList onPizza = this.sauceNCheese.add("onions");
    private StringList pepOnPizza = this.onPizza.add("pepperoni");
    private StringList onPepPizza = this.pepPizza.add("onions");
    private StringList mondoPizza1 =
        this.sauceNCheese.add("artichokes").add("broccoli").add("sausage").add("garlic");
    private StringList mondoPizza2 =
        this.sauceNCheese.add("artichokes").add("onions").add("sausage").add("garlic");
    private StringList tripleOnions =
        this.sauceNCheese.add("onions").add("garlic").add("onions").add("onions");

    /**
     * Run all test methods in the class.
     */
    public static void testEverything ()
    {
        Tester.run (new StringListTest());
    }

        /**
         * Test constructor and getters.
         * 
         * @param  t   the Tester to use
         */
        public void testBasics (Tester t)
        {
            t.checkExpect (this.aList.getFirst(), "Bob");
            t.checkExpect (this.aList.getRest(), new NESL("Phyllis", new ESL()));
            t.checkExpect (((NESL)this.a).getFirst(), "a");
            t.checkExpect (((NESL)this.a).getRest(), this.mt);
            t.checkExpect (((NESL)this.abc).getFirst(), "a");
            NESL bc = (NESL)((NESL)this.abc).getRest();
            t.checkExpect (bc.getFirst(), "b");
            t.checkExpect (((NESL)bc.getRest()).getFirst(), "c");
            t.checkExpect (bc, new NESL("b", new NESL("c", new ESL())));
        }
        
        /**
         * Does the isEmpty method work?
         * 
         * @param  t  the Tester
         */
        public void testIsEmpty (Tester t)
        {
            t.checkExpect (this.mt.isEmpty(), true);
            t.checkExpect (this.a.isEmpty(), false);
            t.checkExpect (((NESL)this.abc).getRest().isEmpty(), false);
        }
        
        /**
         * Does the length method work?
         * 
         * @param  t  the Tester
         * @since v1
         */
        public void testLength (Tester t)
        {
            t.checkExpect (this.mt.length(), 0);
            t.checkExpect (this.a.length(), 1);
            t.checkExpect (this.b.length(), 1);
            t.checkExpect (this.ab.length(), 2);
            t.checkExpect (this.abc.length(), 3);
            t.checkExpect (new NESL("H", new NESL("He", new NESL("Li",
                           new NESL("Be", new NESL("B", new NESL("C",
                           new NESL("N", new NESL("O", new NESL("F",
                           this.mt))))))))).length(), 9);
        }
        
        /**
         * Does the concatAll method work?
         * 
         * @param  t   the Tester
         * @since v2
         */
        public void testConcatAll (Tester t)
        {
            t.checkExpect (this.mt.concatAll(), "");
            t.checkExpect (this.a.concatAll(), "a");
            t.checkExpect (this.b.concatAll(), "b");
            t.checkExpect (this.abc.concatAll(), "abc");
            t.checkExpect (new NESL("H", new NESL("He", new NESL("Li",
                           new NESL("Be", new NESL("B", new NESL("C",
                           new NESL("N", new NESL("O", new NESL("F",
                           this.mt))))))))).concatAll(), "HHeLiBeBCNOF");
        }
        
        /**
         * Does the toString method work?
         * 
         * @param  t  the Tester
         * @since v2
         */
        public void testToString (Tester t)
        {
            t.checkExpect (this.mt.toString(), "[]");
            t.checkExpect (this.a.toString(), "[a,[]]");
            t.checkExpect (this.b.toString(), "[b,[]]");
            t.checkExpect (this.abc.toString(), "[a,[b,[c,[]]]]");
            t.checkExpect (new NESL("H", new NESL("He", new NESL("Li",
                           new NESL("Be", new NESL("B", new NESL("C",
                           new NESL("N", new NESL("O", new NESL("F",
                           this.mt))))))))).toString(), "[H,[He,[Li,[Be,[B,[C,[N,[O,[F,[]]]]]]]]]]");
        }
        
        /**
         * Does the add method work?
         * 
         * @param  t  the Tester
         * @since v3
         */
        public void testAdd (Tester t)
        {
            t.checkExpect (this.mt.add("fnord"), new NESL("fnord", new ESL()));
            t.checkExpect (this.a.add("b"), new NESL("b", new NESL("a", new ESL())));
            t.checkExpect (this.abc.add("snark").add("boojum"),
                           new NESL("boojum", new NESL("snark", new NESL("a",
                           new NESL("b", new NESL("c", new ESL()))))));
        }
        
        /**
         * Does the containsOnions method work?
         * 
         * @param  t  the Tester
         * @since v3
         */
        public void testContainsOnions (Tester t)
        {
            t.checkExpect (this.plainPizza.containsOnions(), false);
            t.checkExpect (this.justSauce.containsOnions(), false);
            t.checkExpect (this.sauceNCheese.containsOnions(), false);
            t.checkExpect (this.pepPizza.containsOnions(), false);
            t.checkExpect (this.onPizza.containsOnions(), true);
            t.checkExpect (this.pepOnPizza.containsOnions(), true);
            t.checkExpect (this.onPepPizza.containsOnions(), true);
            t.checkExpect (this.mondoPizza1.containsOnions(), false);
            t.checkExpect (this.mondoPizza2.containsOnions(), true);
            t.checkExpect (this.tripleOnions.containsOnions(), true);
        }
}
